@@ -24,7 +24,7 @@ use Net::DRI::DRD::ICANN;
 use DateTime::Duration;
 use DateTime;
 
-our $VERSION=do { my @r=(q$Revision: 1.5 $=~/\d+/g); sprintf("%d".".%02d" x $#r, @r); };
+our $VERSION=do { my @r=(q$Revision: 1.6 $=~/\d+/g); sprintf("%d".".%02d" x $#r, @r); };
 
 =pod
 
@@ -69,7 +69,7 @@ See the LICENSE file that comes with this distribution for more details.
 
 #####################################################################################
 
-sub root_servers { return wantarray()? map { $_.'.GTLD-SERVERS.NET' } ('a'..'m') : 'GTLD-SERVERS.NET'; }
+sub root_servers { return wantarray()? map { $_.'.GTLD-SERVERS.NET' } ('a'..'m') : 'GTLD-SERVERS.NET'; } ## TO FIX: return a Hosts object ?
 sub is_thick     { return 0; }
 sub periods      { return map { DateTime::Duration->new(years => $_) } (1..10); }
 sub name         { return "VNDS"; }
@@ -83,6 +83,7 @@ sub transport_protocol_compatible
  my $tn=$to->name();
 
  return 1 if (($pn eq 'RRP') && ($tn eq 'socket_inet'));
+ return 1 if (($pn eq 'EPP') && ($tn eq 'socket_inet'));
  return undef;
 }
 
