@@ -1,6 +1,6 @@
 ## Domain Registry Interface, Stores ordered list of contacts + type (registrant, admin, tech, bill, etc...)
 ##
-## Copyright (c) 2005-2009,2011-2014,2016 Patrick Mevzek <netdri@dotandco.com>. All rights reserved.
+## Copyright (c) 2005 Patrick Mevzek <netdri@dotandco.com>. All rights reserved.
 ##
 ## This file is part of Net::DRI
 ##
@@ -10,16 +10,19 @@
 ## (at your option) any later version.
 ##
 ## See the LICENSE file that comes with this distribution for more details.
+#
+# 
+#
 #########################################################################################
 
+
 package Net::DRI::Data::ContactSet;
 
 use strict;
-use warnings;
 
-use Net::DRI::Exception;
-use Net::DRI::Data::Contact;
-use Net::DRI::Util;
+our @TYPES=('registrant','admin','tech','bill'); ## default list of types
+
+our $VERSION=do { my @r=(q$Revision: 1.2 $=~/\d+/g); sprintf("%d".".%02d" x $#r, @r); };
 
 =pod
 
@@ -33,50 +36,44 @@ This class encapsulates a set of contacts, with associated types. For each type,
 Contacts are compared among themselves by calling the id() method on them. Thus all Contact classes
 must define such a method, which returns a string.
 
-=head1 METHODS
+=over
 
-=head2 new()
+=item *
 
-creates a new object
+C<new()> creates a new object
 
-=head2 types()
+=item *
 
-returns the list of current types stored in this class
+C<types()> returns the list of current types stored in this class
 
-=head2 has_type()
+=item *
 
-returns 1 if the given type as first argument has some contacts in this object, 0 otherwise
+C<has_type()> returns 1 if the given type as first argument has some contacts in this object, 0 otherwise
 
-=head2 add()
+=item *
 
-with the first argument being a contact, and the second (optional) a type, adds the contact
+C<add()> with the first argument being a contact, and the second (optional) a type, adds the contact
 to the list of contacts for this type or all types (if no second argument). If the contact already exists
-(same id()), it will be replaced when found. Returns the object itself.
+(same id()), it will be replaced when found
 
-=head2 del()
+=item *
 
-the opposite of add()
+C<del()> the opposite of add()
 
-=head2 rem()
+=item *
 
-alias for del()
+C<clear()> removes all contact currently associated to all types
 
-=head2 clear()
+=item *
 
-removes all contact currently associated to all types
+C<set()> with an array ref as first argument, and a type (optional) as second, set the current list
+of the given type (or all types) to be the list of contacts in first argument
 
-=head2 set() 
+=item *
 
-with an array ref as first argument, and a type (optional) as second, set the current list
-of the given type (or all types) to be the list of contacts in first argument. Returns the object itself.
+C<get()> returns list (in list context) or first element of list (in scalar context) for the type given as argument
 
-=head2 get()
-
-returns list (in list context) or first element of list (in scalar context) for the type given as argument
-
-=head2 get_all()
-
-returns list of contacts, without duplicates, for all types
+=back
 
 =head1 SUPPORT
 
@@ -96,7 +93,7 @@ Patrick Mevzek, E<lt>netdri@dotandco.comE<gt>
 
 =head1 COPYRIGHT
 
-Copyright (c) 2005-2009,2011-2014,2016 Patrick Mevzek <netdri@dotandco.com>.
+Copyright (c) 2005 Patrick Mevzek <netdri@dotandco.com>.
 All rights reserved.
 
 This program is free software; you can redistribute it and/or modify
@@ -109,51 +106,27 @@ See the LICENSE file that comes with this distribution for more details.
 =cut
 
 ################################################################################################################
-
-our $AUTOLOAD;
-
 sub new
 {
- my ($class,@r)=@_;
- my $self={ c => {} };
- bless $self,$class;
- if (@r)
- {
-  @r=%{$r[0]} if defined $r[0] && ref $r[0] eq 'HASH';
-  Net::DRI::Exception::usererr_invalid_parameters('Invalid number of parameters passed') unless @r%2==0;
-  while(my ($ctype,$ids)=splice(@r,0,2))
-  {
-   my %ccache;
-   foreach my $id (ref $ids eq 'ARRAY' ? @$ids : ($ids))
-   {
-    my $o;
-    if (Net::DRI::Util::isa_contact($id))
-    {
-     $o=$id;
-    } else
-    {
-     $ccache{$id}=Net::DRI::Data::Contact->new()->srid($id) unless exists $ccache{$id};
-     $o=$ccache{$id};
-    }
-    $self->add($o,$ctype);
-   }
-  }
- }
- return $self;
+ my $proto=shift;
+ my $class=ref($proto) || $proto;
+
+ my $self={};
+ $self->{c}={ map { $_ => [] } @TYPES };
+ bless($self,$class);
 }
 
 sub types
 {
  my ($self)=@_;
- my @r=sort { $a cmp $b } grep { @{$self->{c}->{$_}} } keys %{$self->{c}};
- return @r;
+ return sort(grep { @{$self->{c}->{$_}} } keys(%{$self->{c}}));
 }
 
 sub has_type
 {
  my ($self,$ctype)=@_;
- return 0 unless defined $ctype;
- return exists $self->{c}->{$ctype} && @{$self->{c}->{$ctype}} ? 1 : 0;
+ return 0 unless defined($ctype);
+ return exists($self->{c}->{$ctype});
 }
 
 sub is_empty
@@ -168,9 +141,8 @@ sub _pos
  my ($self,$t,$id)=@_;
  my $c=$self->{c};
  my $l=$#{$c->{$t}};
- my @p=grep { my $i=$c->{$t}->[$_]->id(); (defined($i) && ($i eq $id))? 1 : 0 } (0..$l);
- return $p[0] if @p;
- return;
+ my @p=grep { $c->{$t}->[$_]->id() eq $id } (0..$l);
+ return (@p)? $p[0] : undef;
 }
 
 sub add
@@ -180,21 +152,18 @@ sub add
  my $c=$self->{c};
  $c->{$ctype}=[] if (defined($ctype) && !exists($c->{$ctype}));
  my $id=$cobj->id();
- foreach my $k (sort { $a cmp $b } keys %$c)
+ foreach my $k (keys(%$c))
  {
   next if (defined($ctype) && ($k ne $ctype));
-  if ($id)
+  my $p=$self->_pos($k,$id);
+  if (defined($p))
   {
-   my $p=$self->_pos($k,$id);
-   if (defined($p))
-   {
-    $c->{$k}->[$p]=$cobj;
-    next;
-   }
+   $c->{$k}->[$p]=$cobj;
+  } else
+  {
+   push @{$c->{$k}},$cobj;
   }
-  push @{$c->{$k}},$cobj;
  }
- return $self;
 }
 
 sub del
@@ -204,19 +173,15 @@ sub del
  my $c=$self->{c};
  return if (defined($ctype) && !exists($c->{$ctype}));
  my $id=$cobj->id();
- return unless $id;
- foreach my $k (sort { $a cmp $b } keys %$c)
+ foreach my $k (keys(%$c))
  {
   next if (defined($ctype) && ($k ne $ctype));
   my $p=$self->_pos($k,$id);
   next unless defined($p);
   splice(@{$c->{$k}},$p,1);
  }
- return $self;
 }
 
-sub rem { my ($self,@args)=@_; return $self->del(@args); }
-
 sub clear
 {
  my ($self,$ctype)=@_;
@@ -229,29 +194,20 @@ sub set
  return unless defined($robj);
  my $c=$self->{c};
  $c->{$ctype}=[] if (defined($ctype) && !exists($c->{$ctype}));
- foreach my $k (sort { $a cmp $b } keys %$c)
+ foreach my $k (keys(%$c))
  {
   next if (defined($ctype) && ($k ne $ctype));
   $c->{$k}=(ref($robj) eq 'ARRAY')? $robj : [$robj];
  }
- return $self;
 }
 
 sub get
 {
  my ($self,$ctype)=@_;
- return unless defined $ctype;
+ return undef unless defined($ctype);
  my $c=$self->{c};
- return unless exists $c->{$ctype};
- return wantarray ? @{$c->{$ctype}} : $c->{$ctype}->[0];
-}
-
-sub get_all
-{
- my ($self)=@_;
- my %r=map { $_->id() => $_ } map { @{$_} } values(%{$self->{c}});
- my @r=sort { $a cmp $b } keys %r;
- return map { $r{$_} } @r;
+ return undef unless exists($c->{$ctype});
+ return wantarray()? @{$c->{$ctype}} : $c->{$ctype}->[0];
 }
 
 sub match ## compare two contact lists
@@ -263,7 +219,7 @@ sub match ## compare two contact lists
  return 0 unless (keys(%$c1)==keys(%$c2));
  return 0 if grep { ! exists($c1->{$_}) } keys(%$c2);
  return 0 if grep { ! exists($c2->{$_}) } keys(%$c1);
- foreach my $k (sort { $a cmp $b } keys %$c1)
+ foreach my $k (keys(%$c1))
  {
   my %tmp1=map { $_->id() => 1 } @{$c1->{$k}};
   my %tmp2=map { $_->id() => 1 } @{$c2->{$k}};
@@ -281,8 +237,7 @@ sub has_contact
  my $c=$self->{c};
  return 0 if (defined($ctype) && !exists($c->{$ctype}));
  my $id=(ref($cobj))? $cobj->id() : $cobj;
- return 0 unless (defined($id) && $id);
- foreach my $k (sort { $a cmp $b } keys %$c)
+ foreach my $k (keys(%$c))
  {
   next if (defined($ctype) && ($k ne $ctype));
   return 1 if defined($self->_pos($k,$id));
@@ -290,17 +245,5 @@ sub has_contact
  return 0;
 }
 
-sub AUTOLOAD
-{
- my $self=shift;
- my $attr=$AUTOLOAD;
- $attr=~s/.*:://;
- return unless $attr=~m/[^A-Z]/; ## skip DESTROY and all-cap methods
-
- my $ctype;
- Net::DRI::Exception::method_not_implemented($attr,__PACKAGE__) unless ($ctype)=($attr=~m/^contact_(\S+)$/);
- return $self->get($ctype);
-}
-
 ##############################################################################
 1;
