@@ -1,6 +1,6 @@
 ## Domain Registry Interface, AFNIC Web Services Protocol
 ##
-## Copyright (c) 2005,2008-2010,2013 Patrick Mevzek <netdri@dotandco.com>. All rights reserved.
+## Copyright (c) 2005 Patrick Mevzek <netdri@dotandco.com>. All rights reserved.
 ##
 ## This file is part of Net::DRI
 ##
@@ -10,12 +10,14 @@
 ## (at your option) any later version.
 ##
 ## See the LICENSE file that comes with this distribution for more details.
-####################################################################################################
+#
+# 
+#
+#########################################################################################
 
 package Net::DRI::Protocol::AFNIC::WS;
 
 use strict;
-use warnings;
 
 use base qw(Net::DRI::Protocol);
 
@@ -24,6 +26,8 @@ use Net::DRI::Util;
 
 use Net::DRI::Protocol::AFNIC::WS::Message;
 
+our $VERSION=do { my @r=(q$Revision: 1.1 $=~/\d+/g); sprintf("%d".".%02d" x $#r, @r); };
+
 =pod
 
 =head1 NAME
@@ -52,7 +56,7 @@ Patrick Mevzek, E<lt>netdri@dotandco.comE<gt>
 
 =head1 COPYRIGHT
 
-Copyright (c) 2005,2008-2010,2013 Patrick Mevzek <netdri@dotandco.com>.
+Copyright (c) 2005 Patrick Mevzek <netdri@dotandco.com>.
 All rights reserved.
 
 This program is free software; you can redistribute it and/or modify
@@ -64,25 +68,39 @@ See the LICENSE file that comes with this distribution for more details.
 
 =cut
 
-####################################################################################################
+###################################################################################################
 
 sub new
 {
- my ($c,$ctx,$rp)=@_;
- my $self=$c->SUPER::new($ctx);
+ my $h=shift;
+ my $c=ref($h) || $h;
+
+ my ($drd,$version,$extrah)=@_;
+
+ my $self=$c->SUPER::new(); ## we are now officially a Net::DRI::Protocol object
  $self->name('afnic_ws');
- $self->version('0.1');
- $self->factories('message',sub { my $m=Net::DRI::Protocol::AFNIC::WS::Message->new(); $m->version('0.1'); return $m; });
- $self->_load($rp);
+ $self->version($VERSION);
+
+ $self->capabilities({});
+
+ $self->factories({ 'message' => 'Net::DRI::Protocol::AFNIC::WS::Message',
+                  });
+
+ bless($self,$c); ## rebless
+
+ $self->_load($extrah);
  return $self;
 }
 
 sub _load
 {
- my ($self,$rp)=@_;
- my @class=map { 'Net::DRI::Protocol::AFNIC::WS::'.$_ } ('Domain');
- return $self->SUPER::_load(@class);
+ my ($self,$extrah)=@_;
+
+ my @class=map { "Net::DRI::Protocol::AFNIC::WS::".$_ } ('Domain');
+
+ $self->SUPER::_load(@class);
 }
 
-####################################################################################################
+
+##########################################################################################################
 1;
