@@ -1,6 +1,6 @@
 ## Domain Registry Interface, EPP Status
 ##
-## Copyright (c) 2005,2007,2008,2010,2014 Patrick Mevzek <netdri@dotandco.com>. All rights reserved.
+## Copyright (c) 2005 Patrick Mevzek <netdri@dotandco.com>. All rights reserved.
 ##
 ## This file is part of Net::DRI
 ##
@@ -10,18 +10,21 @@
 ## (at your option) any later version.
 ##
 ## See the LICENSE file that comes with this distribution for more details.
-####################################################################################################
+#
+# 
+#
+#########################################################################################
 
 package Net::DRI::Protocol::EPP::Core::Status;
 
 use base qw!Net::DRI::Data::StatusList!;
 
-use utf8;
 use strict;
-use warnings;
 
 use Net::DRI::Exception;
 
+our $VERSION=do { my @r=(q$Revision: 1.1 $=~/\d+/g); sprintf("%d".".%02d" x $#r, @r); };
+
 =pod
 
 =head1 NAME
@@ -50,7 +53,7 @@ Patrick Mevzek, E<lt>netdri@dotandco.comE<gt>
 
 =head1 COPYRIGHT
 
-Copyright (c) 2005,2007,2008,2010,2014 Patrick Mevzek <netdri@dotandco.com>.
+Copyright (c) 2005 Patrick Mevzek <netdri@dotandco.com>.
 All rights reserved.
 
 This program is free software; you can redistribute it and/or modify
@@ -62,12 +65,15 @@ See the LICENSE file that comes with this distribution for more details.
 
 =cut
 
-####################################################################################################
+#######################################################################################
 
 sub new
 {
- my $class=shift;
+ my $proto=shift;
+ my $class=ref($proto) || $proto;
+
  my $self=$class->SUPER::new('epp','1.0');
+ bless($self,$class);
 
  my %s=('delete'   => 'clientDeleteProhibited',
         'renew'    => 'clientRenewProhibited',
@@ -78,49 +84,67 @@ sub new
  $self->_register_pno(\%s);
 
  my $msg=shift;
- return $self unless defined $msg;
- Net::DRI::Exception::err_invalid_parameters() unless ref $msg eq 'ARRAY';
- $self->add(@$msg);
- return $self;
-}
+ return $self unless defined($msg);
 
-sub is_core_status
-{
- return (shift=~m/^client(?:Hold|(?:Delete|Renew|Update|Transfer)Prohibited)$/);
+ if (ref($msg) eq 'ARRAY')
+ {
+  $self->add(@$msg);
+ } else
+ {
+  Net::DRI::Exception::err_invalid_parameters();
+ }
+
+ return $self;
 }
 
 sub build_xml
 {
- my ($self,$name,$range)=@_;
- $range='core' unless defined($range);
+ my ($self,$name)=@_;
  my @d;
  my $rd=$self->status_details();
  while(my ($k,$v)=each(%$rd))
  {
-  next if (($range eq 'core') xor is_core_status($k));
-  if ($v && ref $v eq 'HASH' && keys %$v)
+  if ($v && ref($v) && keys(%$v))
   {
    my %tmp=(s => $k);
-   $tmp{lang}=$v->{lang} if exists $v->{lang};
+   $tmp{lang}=$v->{lang} if exists($v->{lang});
    push @d,[$name,$v->{msg} || '',\%tmp];
   } else
   {
-   push @d,[$name,{s=>$k}];
+   push @d,[$name,undef,{s=>$k}];
   }
  }
  return @d;
 }
 
 sub is_active    { return shift->has_any('ok'); }
-sub is_published { return shift->has_not(qw/clientHold serverHold inactive/); }
-sub is_pending   { return shift->has_any(qw/pendingCreate pendingDelete pendingRenew pendingTransfer pendingUpdate/); }
+sub is_published { return shift->has_not('clientHold','serverHold','inactive'); }
+sub is_pending   { return shift->has_any('pendingCreate','pendingDelete','pendingRenew','pendingTransfer','pendingUpdate'); }
 sub is_linked    { return shift->has_any('linked'); }
-sub is_grace     { return shift->has_any(qw/addPeriod autoRenewPeriod renewPeriod transferPeriod redemptionPeriod pendingRestore pendingDelete/); } ## defined in RFC3915 §3.1
 
-sub can_delete   { my $self=shift; return (!$self->is_linked() && !$self->is_pending() && $self->has_not(qw/clientDeleteProhibited serverDeleteProhibited/))? 1 : 0; }
-sub can_renew    { my $self=shift; return (!$self->is_pending() && $self->has_not(qw/clientRenewProhibited serverRenewProhibited/))? 1 : 0; }
-sub can_update   { my $self=shift; return (!$self->is_pending() && $self->has_not(qw/clientUpdateProhibited serverUpdateProhibited/))? 1 : 0; }
-sub can_transfer { my $self=shift; return (!$self->is_pending() && $self->has_not(qw/clientTransferProhibited serverTransferProhibited/))? 1 : 0; }
+sub can_delete
+{
+ my $self=shift;
+ my $ispending=$self->is_pending();
+ return !$ispending && $self->has_not('clientDeleteProhibited','serverDeleteProhibited');
+}
 
-####################################################################################################
+sub can_renew
+{
+ my $self=shift;
+ my $ispending=$self->is_pending();
+ return !$ispending && $self->has_not('clientRenewProhibited','serverRenewProhibited');
+}
+
+sub can_update
+{
+ my $self=shift;
+ my $ispending=$self->is_pending();
+ return !$ispending && $self->has_not('clientUpdateProhibited','serverUpdateProhibited');
+}
+
+sub can_transfer { return shift->has_not('clientTransferProhibited','serverTransferProhibited'); }
+
+
+####################################################################################################################
 1;
