@@ -1,6 +1,6 @@
 ## Domain Registry Interface, EPP Grace Period commands (RFC3915)
 ##
-## Copyright (c) 2005,2006,2008-2010,2013,2016,2018-2019 Patrick Mevzek <netdri@dotandco.com>. All rights reserved.
+## Copyright (c) 2005 Patrick Mevzek <netdri@dotandco.com>. All rights reserved.
 ##
 ## This file is part of Net::DRI
 ##
@@ -10,16 +10,21 @@
 ## (at your option) any later version.
 ##
 ## See the LICENSE file that comes with this distribution for more details.
-####################################################################################################
+#
+# 
+#
+#########################################################################################
 
 package Net::DRI::Protocol::EPP::Extensions::GracePeriod;
 
 use strict;
-use warnings;
 
 use Net::DRI::Util;
 use Net::DRI::Exception;
 
+our $VERSION=do { my @r=(q$Revision: 1.1 $=~/\d+/g); sprintf("%d".".%02d" x $#r, @r); };
+our $NS='urn:ietf:params:xml:ns:rgp-1.0';
+
 =pod
 
 =head1 NAME
@@ -48,7 +53,7 @@ Patrick Mevzek, E<lt>netdri@dotandco.comE<gt>
 
 =head1 COPYRIGHT
 
-Copyright (c) 2005,2006,2008-2010,2013,2016,2018-2019 Patrick Mevzek <netdri@dotandco.com>.
+Copyright (c) 2005 Patrick Mevzek <netdri@dotandco.com>.
 All rights reserved.
 
 This program is free software; you can redistribute it and/or modify
@@ -60,7 +65,8 @@ See the LICENSE file that comes with this distribution for more details.
 
 =cut
 
-####################################################################################################
+
+##########################################################
 
 sub register_commands
 {
@@ -73,16 +79,8 @@ sub register_commands
  return { 'domain' => \%tmp };
 }
 
-sub capabilities_add { return ('domain_update','rgp',['set']); }
+##################################################################################################
 
-sub setup
-{
- my ($class,$po,$version)=@_;
- $po->ns({ 'rgp' => 'urn:ietf:params:xml:ns:rgp-1.0' });
- return;
-}
-
-####################################################################################################
 ########### Query commands
 
 sub info_parse
@@ -91,17 +89,15 @@ sub info_parse
  my $mes=$po->message();
  return unless $mes->is_success();
 
- my $ns=$mes->ns('rgp');
- my $infdata=$mes->get_extension('rgp', 'infData');
- return unless defined $infdata;
+ my $infdata=$mes->get_content('infData',$NS,1);
+ return unless $infdata;
 
  my $cs=$rinfo->{domain}->{$oname}->{status}; ## a Net::DRI::Protocol::EPP::Core::Status object
 
- foreach my $el ($infdata->getChildrenByTagNameNS($ns,'rgpStatus'))
+ foreach my $el ($infdata->getElementsByTagNameNS($NS,'rgpStatus'))
  {
   $cs->add($el->getAttribute('s'));
  }
- return;
 }
 
 ############ Transform commands
@@ -109,19 +105,23 @@ sub info_parse
 sub update
 {
  my ($epp,$domain,$todo)=@_;
+ my $mes=$epp->message();
 
  my $rgp=$todo->set('rgp');
- return unless (defined $rgp && $rgp && ref $rgp eq 'HASH');
+ return unless (defined($rgp) && $rgp && (ref($rgp) eq 'HASH'));
 
  my $op=$rgp->{op} || '';
  Net::DRI::Exception::usererr_invalid_parameters('RGP op must be request or report') unless ($op=~m/^(?:request|report)$/);
- Net::DRI::Exception::usererr_invalid_parameters('Report data must be included if the operation is a report') unless (($op eq 'request') xor exists $rgp->{report});
+ Net::DRI::Exception::usererr_invalid_parameters('Report data must be included if the operation is a report') unless (($op eq 'request') xor exists($rgp->{report}));
 
- my @restore = ({ op => $op });  # this is enough for op=request
- if ($op eq 'report')
+ my $eid=$mes->command_extension_register('rgp:update','xmlns:rgp="urn:ietf:params:xml:ns:rgp-1.0" xsi:schemaLocation="urn:ietf:params:xml:ns:rgp-1.0 rgp-1.0.xsd"');
+
+ if ($op eq 'request')
+ {
+  $mes->command_extension($eid,['rgp:restore',undef,{ op => $op }]);
+ } else
  {
   my %r=%{$rgp->{report}};
-  my $def=$epp->default_parameters();
   my @d;
   push @d,['rgp:preData',$r{predata}]; ## XML data is possible in the RFC, but not here ?!
   push @d,['rgp:postData',$r{postdata}]; ## ditto
@@ -131,14 +131,23 @@ sub update
   Net::DRI::Util::check_isa($r{restime},'DateTime');
   push @d,['rgp:resTime',$r{restime}->strftime('%Y-%m-%dT%T.%1NZ')];
   push @d,['rgp:resReason',$r{reason}];
-  push @d,['rgp:statement',$r{statement1},exists $r{statement1_lang} ? {lang => $r{statement1_lang}} : ()];
-  push @d,['rgp:statement',$r{statement2},exists $r{statement2_lang} ? {lang => $r{statement2_lang}} : ()];
-  push @d,['rgp:other',$r{other}] if exists $r{other};
-  push @restore, ['rgp:report', @d];
+  if (exists($r{statement1_lang}))
+  {
+   push @d,['rgp:statement',$r{statement1},{lang => $r{statement1_lang}}];
+  } else
+  {
+   push @d,['rgp:statement',$r{statement1}];
+  }
+  if (exists($r{statement2_lang}))
+  {
+   push @d,['rgp:statement',$r{statement2},{lang => $r{statement2_lang}}];
+  } else
+  {
+   push @d,['rgp:statement',$r{statement2}];
+  }
+  push @d,['rgp:other',$r{other}] if exists($r{other});
+  $mes->command_extension($eid,['rgp:restore',[['rgp:report',\@d]],{ op => $op }]);
  }
-
- $epp->message()->command_extension('rgp', ['update', ['restore', @restore]]);
- return;
 }
 
 sub update_parse
@@ -147,13 +156,12 @@ sub update_parse
  my $mes=$po->message();
  return unless $mes->is_success();
 
- my $updata=$mes->get_extension('rgp', 'upData');
- return unless defined $updata;
+ my $updata=$mes->get_content('upData',$NS,1);
+ return unless $updata;
 
  ## We do nothing, since the rgpStatus alone is useless
  ## (we do not have the other status)
- return;
 }
 
-####################################################################################################
+#########################################################################################################
 1;
