@@ -20,7 +20,7 @@ package Net::DRI::Protocol::RRP::Connection;
 use strict;
 use Net::DRI::Protocol::RRP::Message;
 
-our $VERSION=do { my @r=(q$Revision: 1.5 $=~/\d+/g); sprintf("%d".".%02d" x $#r, @r); };
+our $VERSION=do { my @r=(q$Revision: 1.9 $=~/\d+/g); sprintf("%d".".%02d" x $#r, @r); };
 
 =pod
 
@@ -69,7 +69,7 @@ See the LICENSE file that comes with this distribution for more details.
 sub login
 {
  shift if ($_[0] eq __PACKAGE__);
- my ($id,$pass)=@_;
+ my ($id,$pass,$cltrid,$dr)=@_;
  my $mes=Net::DRI::Protocol::RRP::Message->new({ command => 'session', options => { Id => $id, Password => $pass}});
  return $mes->as_string();
 }
@@ -88,39 +88,54 @@ sub keepalive
 
 ########################################################################
 
-sub is_login_successfull
+sub get_data
 {
  shift if ($_[0] eq __PACKAGE__);
- my $dc=shift; ## a DataRaw object
- my $code=find_code($dc);
- return (defined($code) && ($code==200));
+ my ($to,$sock)=@_;
+
+ my (@l);
+ while(my $l=$sock->getline())
+ {
+  push @l,$l;
+  last if ($l=~m/^\.\s*\n?$/);
+ }
+ die() unless ($l[-1]=~m/^\.\s*\n?$/);
+ return Net::DRI::Data::Raw->new_from_array(\@l);
 }
 
-sub is_end_command
+sub is_greeting_successful
 {
  shift if ($_[0] eq __PACKAGE__);
- my $dc=shift; ## a DataRaw object
- my $ll=(ref($dc) eq 'ARRAY')? $dc->[-1] : $dc->last_line();
- return ($ll=~m/^\.\s*\n?$/)? 1 : 0;
+ my $dc=shift;
+ my $code=find_code($dc);
+ return ($code==0)? 1 : 0;
+}
+
+sub is_login_successful
+{
+ shift if ($_[0] eq __PACKAGE__);
+ my $dc=shift;
+ my $code=find_code($dc);
+ return (defined($code) && ($code==200));
 }
 
 sub is_server_close
 {
  shift if ($_[0] eq __PACKAGE__);
- my $dc=shift; ## a DataRaw object
+ my $dc=shift;
  my $code=find_code($dc);
 
- ## 220 : after a successfull QUIT
+ ## 220 : after a successful QUIT
  ## 420 : Command failed due to server error. Server closing connection
  ## 520 : Server closing connection. Client should try opening new connection (timeout)
  ## 521 : Too many sessions open. Server closing connection
- return (defined($code) && ($code=~m/^([245]20|521)$/));
+ return (defined($code) && ($code=~m/^(?:[245]20|521)$/));
 }
 
 sub find_code
 {
- my $dc=shift; ## a DataRaw object
- my @a=(ref($dc) eq 'ARRAY')? @$dc : $dc->as_array();
+ my $dc=shift;
+ my @a=$dc->as_array();
  return 0 if ($a[0]=~m/^.+ RRP Server version/); ## initial login
  return undef unless $#a>0; ## at least 2 lines
  return undef unless $a[-1]=~m/^\.\s*\n?$/;
